@@ -4,7 +4,6 @@ mod compile;
 mod config;
 mod run;
 mod settings;
-mod wasm2obj;
 mod wast;
 
-pub use self::{compile::*, config::*, run::*, settings::*, wasm2obj::*, wast::*};
+pub use self::{compile::*, config::*, run::*, settings::*, wast::*};
